package com.ruoyi.system.controller.user;

import org.springframework.web.bind.annotation.DeleteMapping;
import java.util.Iterator;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.web.bind.annotation.PutMapping;
import java.util.Objects;
import com.ruoyi.system.enums.BussLogEnum;
import com.ruoyi.common.core.utils.ServletUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.validation.annotation.Validated;
import java.util.function.Function;
import com.ruoyi.system.api.domain.SysRole;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import com.ruoyi.common.security.annotation.InnerAuth;
import java.util.Set;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.system.api.model.LoginUser;
import com.ruoyi.common.core.domain.R;
import org.springframework.web.bind.annotation.PathVariable;
import java.io.IOException;
import com.ruoyi.common.security.utils.SecurityUtils;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.bind.annotation.PostMapping;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.core.utils.poi.ExcelUtil;
import javax.servlet.http.HttpServletResponse;
import com.ruoyi.common.core.web.domain.AjaxResult;
import org.springframework.web.bind.annotation.GetMapping;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import java.util.List;
import com.ruoyi.common.core.web.page.TableDataInfo;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.service.ISysBussLogService;
import com.ruoyi.common.security.service.TokenService;
import com.ruoyi.system.service.ISnTemplatePermsService;
import com.ruoyi.system.service.ISysPermissionService;
import com.ruoyi.system.service.ISysPostService;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import com.ruoyi.system.service.ISysUserService;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.core.web.controller.BaseController;

@RestController
@RequestMapping({ "/agent" })
public class SysAgentController extends BaseController
{
    @Autowired
    private ISysUserService userService;
    @Autowired
    private ISysRoleService roleService;
    @Autowired
    private ISysDeptService deptService;
    @Autowired
    private ISysPostService postService;
    @Autowired
    private ISysPermissionService permissionService;
    @Autowired
    private ISnTemplatePermsService templatePermsService;
    @Autowired
    private TokenService tokenService;
    @Autowired
    private ISysBussLogService sysBussLogService;
    
    @RequiresPermissions({ "system:agent:list" })
    @GetMapping({ "/list" })
    public TableDataInfo list(final SysUser user) {
        if (user.getParams().containsKey("isPage")) {
            if ("true".equalsIgnoreCase(user.getParams().get("isPage").toString())) {
                this.startPage();
            }
        }
        else {
            this.startPage();
        }
        user.setUserType("agent");
        final List<SysUser> list = this.userService.selectUserList(user);
        return this.getDataTable((List)list);
    }
    
    @GetMapping({ "/selectList" })
    public AjaxResult selectList(final SysUser user) {
        user.setUserType("agent");
        return AjaxResult.success((Object)this.userService.selectList(user));
    }
    
    @Log(title = "代理管理", businessType = BusinessType.EXPORT)
    @RequiresPermissions({ "system:agent:export" })
    @PostMapping({ "/export" })
    public void export(final HttpServletResponse response, final SysUser user) {
        final List<SysUser> list = this.userService.selectUserList(user);
        final ExcelUtil<SysUser> util = (ExcelUtil<SysUser>)new ExcelUtil((Class)SysUser.class);
        util.exportExcel(response, (List)list, "代理数据");
    }
    
    @Log(title = "代理管理", businessType = BusinessType.IMPORT)
    @RequiresPermissions({ "system:agent:import" })
    @PostMapping({ "/importData" })
    public AjaxResult importData(final MultipartFile file, final boolean updateSupport) throws Exception {
        final ExcelUtil<SysUser> util = (ExcelUtil<SysUser>)new ExcelUtil((Class)SysUser.class);
        final List<SysUser> userList = util.importExcel(file.getInputStream());
        final String operName = SecurityUtils.getUsername();
        final String message = this.userService.importUser((List)userList, Boolean.valueOf(updateSupport), operName);
        return this.success(message);
    }
    
    @PostMapping({ "/importTemplate" })
    public void importTemplate(final HttpServletResponse response) throws IOException {
        final ExcelUtil<SysUser> util = (ExcelUtil<SysUser>)new ExcelUtil((Class)SysUser.class);
        util.importTemplateExcel(response, "代理数据");
    }
    
    @InnerAuth
    @GetMapping({ "/info/{username}" })
    public R<LoginUser> info(@PathVariable("username") final String username) {
        final SysUser sysUser = this.userService.selectUserByUserName(username);
        if (StringUtils.isNull((Object)sysUser)) {
            return (R<LoginUser>)R.fail("代理名或密码错误");
        }
        final Set<String> roles = this.permissionService.getRolePermission(sysUser);
        final Set<String> permissions = this.permissionService.getMenuPermission(sysUser);
        final LoginUser sysUserVo = new LoginUser();
        sysUserVo.setSysUser(sysUser);
        sysUserVo.setRoles((Set)roles);
        sysUserVo.setPermissions((Set)permissions);
        return (R<LoginUser>)R.ok((Object)sysUserVo);
    }
    
    @GetMapping({ "getInfo" })
    public AjaxResult getInfo() {
        final SysUser user = this.userService.selectUserById(SecurityUtils.getUserId());
        final Set<String> roles = this.permissionService.getRolePermission(user);
        final Set<String> permissions = this.permissionService.getMenuPermission(user);
        final AjaxResult ajax = AjaxResult.success();
        ajax.put("user", (Object)user);
        ajax.put("roles", (Object)roles);
        ajax.put("permissions", (Object)permissions);
        return ajax;
    }
    
    @RequiresPermissions({ "system:agent:query" })
    @GetMapping({ "/", "/{userId}" })
    public AjaxResult getInfo(@PathVariable(value = "userId", required = false) final Long userId) {
        final AjaxResult ajax = AjaxResult.success();
        final List<SysRole> roles = this.roleService.selectRoleAll();
        ajax.put("roles", (Object)roles.stream().filter(r -> r.getRoleKey().equals("agent")).collect((Collector<? super Object, ?, List<? super Object>>)Collectors.toList()));
        ajax.put("posts", (Object)this.postService.selectPostAll());
        if (StringUtils.isNotNull((Object)userId)) {
            final SysUser sysUser = this.userService.selectUserById(userId);
            ajax.put("data", (Object)sysUser);
            ajax.put("postIds", (Object)this.postService.selectPostListByUserId(userId));
            ajax.put("roleIds", sysUser.getRoles().stream().map(SysRole::getRoleId).collect(Collectors.toList()));
        }
        return ajax;
    }
    
    @RequiresPermissions({ "system:agent:add" })
    @Log(title = "代理管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody final SysUser user) {
        if (!this.userService.checkUserNameUnique(user)) {
            return this.error("新增代理'" + user.getUserName() + "'失败，登录账号已存在");
        }
        if (StringUtils.isNotEmpty(user.getPhonenumber()) && !this.userService.checkPhoneUnique(user)) {
            return this.error("新增代理'" + user.getUserName() + "'失败，手机号码已存在");
        }
        if (StringUtils.isNotEmpty(user.getEmail()) && !this.userService.checkEmailUnique(user)) {
            return this.error("新增代理'" + user.getUserName() + "'失败，邮箱账号已存在");
        }
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        final SysUser curUser = this.userService.selectUserById(loginUser.getUserid());
        if (curUser.getServerLimit() < user.getServerLimit()) {
            return this.error("您当前最多可分配" + curUser.getServerLimit() + "个服务数量");
        }
        if (StringUtils.isEmpty(curUser.getCloakId())) {
            return this.error("CloakId未配置，请联系管理员");
        }
        user.setCloakId(curUser.getCloakId());
        user.setUserType("agent");
        user.setCreateBy(SecurityUtils.getUserId().toString());
        user.setCreateName(SecurityUtils.getLoginUser().getUsername());
        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
        user.setDealerId(SecurityUtils.getUserId());
        user.setDealerName(SecurityUtils.getLoginUser().getUsername());
        final int num = this.userService.insertUser(user);
        if (num > 0) {
            this.sysBussLogService.insertSysBussLog(BussLogEnum.AddAgent.getType(), user.getUserId(), user.getUserName(), "添加代理 " + user.getUserName());
        }
        if (user.getParams().containsKey("templateIds")) {
            try {
                final List<Integer> templateIds = user.getParams().get("templateIds");
                this.templatePermsService.updateUserPerms((List)templateIds.stream().map((Function<? super Object, ?>)Integer::longValue).collect((Collector<? super Object, ?, List<? super Object>>)Collectors.toList()), user.getUserId());
            }
            catch (final Exception ex) {}
        }
        return this.toAjax(num);
    }
    
    @RequiresPermissions({ "system:agent:edit" })
    @Log(title = "代理管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody final SysUser user) {
        this.userService.checkUserAllowed(user);
        this.userService.checkUserDataScope(user.getUserId());
        this.deptService.checkDeptDataScope(user.getDeptId());
        this.roleService.checkRoleDataScope(user.getRoleIds());
        if (!this.userService.checkUserNameUnique(user)) {
            return this.error("修改代理'" + user.getUserName() + "'失败，登录账号已存在");
        }
        if (StringUtils.isNotEmpty(user.getPhonenumber()) && !this.userService.checkPhoneUnique(user)) {
            return this.error("修改代理'" + user.getUserName() + "'失败，手机号码已存在");
        }
        if (StringUtils.isNotEmpty(user.getEmail()) && !this.userService.checkEmailUnique(user)) {
            return this.error("修改代理'" + user.getUserName() + "'失败，邮箱账号已存在");
        }
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        final SysUser obj = this.userService.selectUserById(user.getUserId());
        final SysUser dealer = this.userService.selectUserById(obj.getDealerId());
        if (dealer.getServerLimit() >= user.getServerLimit()) {
            if (user.getParams().containsKey("templateIds")) {
                try {
                    final List<Integer> templateIds = user.getParams().get("templateIds");
                    this.templatePermsService.updateUserPerms((List)templateIds.stream().map((Function<? super Object, ?>)Integer::longValue).collect((Collector<? super Object, ?, List<? super Object>>)Collectors.toList()), user.getUserId());
                }
                catch (final Exception ex) {}
            }
            if (!Objects.equals(user.getServerLimit(), obj.getServerLimit())) {
                this.sysBussLogService.insertSysBussLog(BussLogEnum.OptServerNum.getType(), obj.getUserId(), obj.getUserName(), "服务数量变更 " + obj.getServerLimit() + " => " + user.getServerLimit());
            }
            user.setUpdateBy(SecurityUtils.getUsername());
            return this.toAjax(this.userService.updateUser(user));
        }
        final Set<String> roles = loginUser.getRoles();
        if (roles.contains("dealer")) {
            return this.error("您当前最多可分配" + dealer.getServerLimit() + "个服务数量");
        }
        return this.error("服务数量不能大于经销商的服务数量");
    }
    
    @RequiresPermissions({ "system:agent:remove" })
    @Log(title = "代理管理", businessType = BusinessType.DELETE)
    @DeleteMapping({ "/{userIds}" })
    public AjaxResult remove(@PathVariable final Long[] userIds) {
        if (ArrayUtils.contains(userIds, SecurityUtils.getUserId())) {
            return this.error("当前代理不能删除");
        }
        final SysUser param = new SysUser();
        param.getParams().put("userIds", userIds);
        final List<SysUser> users = this.userService.selectUserList(param);
        final int num = this.userService.deleteUserByIds(userIds);
        for (final SysUser user : users) {
            this.sysBussLogService.insertSysBussLog(BussLogEnum.DelAgent.getType(), user.getUserId(), user.getUserName(), "删除代理 " + user.getUserName());
        }
        return this.toAjax(num);
    }
    
    @RequiresPermissions({ "system:agent:resetPwd" })
    @Log(title = "代理管理", businessType = BusinessType.UPDATE)
    @PutMapping({ "/resetPwd" })
    public AjaxResult resetPwd(@RequestBody final SysUser user) {
        this.userService.checkUserAllowed(user);
        this.userService.checkUserDataScope(user.getUserId());
        user.setUpdateBy(SecurityUtils.getUsername());
        return this.toAjax(this.userService.resetPwd(user));
    }
    
    @RequiresPermissions({ "system:agent:resetPwd" })
    @Log(title = "代理管理", businessType = BusinessType.UPDATE)
    @PutMapping({ "/changeStatus" })
    public AjaxResult changeStatus(@RequestBody final SysUser user) {
        this.userService.checkUserAllowed(user);
        this.userService.checkUserDataScope(user.getUserId());
        user.setUpdateBy(SecurityUtils.getUsername());
        return this.toAjax(this.userService.updateUserStatus(user));
    }
}
